#!/usr/bin/python
# Authors:
#     Endi S. Dewata <edewata@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2015 Red Hat, Inc.
# All rights reserved.
#

import base64
import ldap
import ldap.filter
import nss.nss as nss
import os


class PKISubsystem(object):

    def __init__(self, dir):

        self.dir = dir
        self.cs_conf = os.path.join(self.dir, 'conf', 'CS.cfg')
        self.password_conf = os.path.join(self.dir, 'conf', 'password.conf')

        self.config = {}
        self.passwords = {}

        self.type = None
        self.prefix = None

        self.load()

    def load(self):

        self.config = {}

        lines =  open(self.cs_conf).read().splitlines()

        for line in lines:
            parts = line.split('=', 1)
            name = parts[0]
            value = parts[1]
            self.config[name] = value

        self.type = self.config['cs.type']
        self.prefix = self.type.lower()

        self.passwords = {}

        lines =  open(self.password_conf).read().splitlines()

        for line in lines:
            parts = line.split('=', 1)
            name = parts[0]
            value = parts[1]
            self.passwords[name] = value

    def get_system_certs(self):

        tags = self.config['%s.cert.list' % self.prefix].split(',')

        certs = []

        for tag in tags:

            cert = self.get_system_cert(tag)

            certs.append(cert)

        return certs

    def get_system_cert(self, tag):

        cert = {}
        cert['tag'] = tag
        cert['nickname'] = self.config['%s.cert.%s.nickname' % (self.prefix, tag)]
        cert['usage'] = self.config['%s.cert.%s.certusage' % (self.prefix, tag)]
        cert['token'] = self.config['%s.%s.tokenname' % (self.prefix, tag)]
        cert['certificate'] = base64.b64decode(self.config['%s.%s.cert' % (self.prefix, tag)])
        cert['request'] = base64.b64decode(self.config['%s.%s.certreq' % (self.prefix, tag)])

        if not nss.nss_is_initialized():
            nss.nss_init_nodb()

        nss_cert = nss.Certificate(buffer(cert['certificate']))
        cert['version'] = nss_cert.version
        cert['serial_number'] = nss_cert.serial_number
        cert['issuer'] = nss_cert.issuer
        cert['subject'] = nss_cert.subject

        return cert

    def open_database(self):

        hostname = self.config['internaldb.ldapconn.host']
        port = self.config['internaldb.ldapconn.port']
        base_dn = self.config['internaldb.basedn']
        bind_dn = self.config['internaldb.ldapauth.bindDN']

        bind_password = self.passwords['internaldb']

        con = ldap.initialize('ldap://%s:%s' % (hostname, port))
        con.simple_bind_s(bind_dn, bind_password)

        return con

    def create_user_object(self, entry):

        user = {}
        attrs = entry[1]

        user['uid'] = attrs['uid'][0]
        user['cn'] = attrs['cn'][0]
        user['sn'] = attrs['sn'][0]
        user['type'] = attrs['usertype'][0]

        if 'description' in attrs:
            user['description'] = attrs['description'][0]

        if 'mail' in attrs:
            user['mail'] = attrs['mail'][0]

        if 'userCertificate' in attrs:
             user['certificate'] = attrs['userCertificate'][0]

        if 'userPassword' in attrs:
            user['password'] = attrs['userPassword'][0]

        if 'userstate' in attrs:
            user['state'] = attrs['userstate'][0]

        return user

    def create_user_attrs(self, user):

        attrs = [
            ('objectclass', [
                'top',
                'person',
                'organizationalPerson',
                'inetOrgPerson',
                'cmsUser'
            ]),
            ('uid', [user['uid']]),
            ('cn', [user['cn']]),
            ('sn', [user['sn']]),
            ('usertype', [user['type']]),
        ]

        if 'description' in user:
            attrs.append(('description', [user['description']]))

        if 'mail' in user:
            attrs.append(('mail', [user['mail']]))

        if 'certificate' in user:
            attrs.append(('userCertificate', [user['certificate']]))

        if 'password' in user:
            attrs.append(('userPassword', [user['password']]))

        if 'state' in user:
            attrs.append(('userstate', [user['state']]))

        return attrs

    def add_mod_tuple(self, modlist, name, attr, changes):
        if name in changes:
            if changes[name]:
                modlist.append((ldap.MOD_REPLACE, attr, [changes[name]]))
            else:
                modlist.append((ldap.MOD_DELETE, attr, None))

    def create_user_modlist(self, changes):

        modlist = []

        self.add_mod_tuple(modlist, 'cn', 'cn', changes)
        self.add_mod_tuple(modlist, 'sn', 'sn', changes)
        self.add_mod_tuple(modlist, 'type', 'usertype', changes)
        self.add_mod_tuple(modlist, 'description', 'description', changes)
        self.add_mod_tuple(modlist, 'mail', 'mail', changes)
        self.add_mod_tuple(modlist, 'certificate', 'userCertificate', changes)
        self.add_mod_tuple(modlist, 'password', 'userPassword', changes)
        self.add_mod_tuple(modlist, 'state', 'userstate', changes)

        return modlist

    def find_system_users(self, description=None, certificate=None):

        base_dn = self.config['internaldb.basedn']

        filter = ''
        counter = 0

        if description:
            escaped_value = ldap.filter.escape_filter_chars(description)
            filter = filter + '(description=%s)' % escaped_value
            counter = counter + 1

        if certificate:
            escaped_value = ldap.filter.escape_filter_chars(certificate)
            filter = filter + '(userCertificate=%s)' % escaped_value
            counter = counter + 1

        if counter == 0:
            filter = 'objectClass=*'

        elif counter == 1:
            pass

        else:
            filter = '(&' + filter + ')'

        con = self.open_database()

        entries = con.search_s(
            'ou=people,%s' % base_dn,
            ldap.SCOPE_ONELEVEL,
            filter,
            None)

        con.unbind_s()

        users = []
        for entry in entries:
            users.append(self.create_user_object(entry))

        return users

    def get_system_user(self, user_id):

        base_dn = self.config['internaldb.basedn']

        con = self.open_database()

        entries = con.search_s(
            'uid=%s,ou=people,%s' % (user_id, base_dn),
            ldap.SCOPE_BASE,
            '(objectclass=*)',
            None)

        con.unbind_s()

        entry = entries[0]

        return self.create_user_object(entry)

    def add_system_user(self, user):

        base_dn = self.config['internaldb.basedn']
        dn = 'uid=%s,ou=people,%s' % (user['uid'], base_dn)

        attrs = self.create_user_attrs(user)

        con = self.open_database()

        con.add_s(dn, attrs)

        con.unbind_s()

    def modify_system_user(self, user_id, changes):

        base_dn = self.config['internaldb.basedn']
        dn = 'uid=%s,ou=people,%s' % (user_id, base_dn)

        modlist = self.create_user_modlist(changes)

        con = self.open_database()

        con.modify_s(dn, modlist)

        con.unbind_s()

    def remove_system_user(self, user_id):

        base_dn = self.config['internaldb.basedn']

        con = self.open_database()

        con.delete_s(
            'uid=%s,ou=people,%s' % (user_id, base_dn))

        con.unbind_s()

    def create_group_from_ldap_entry(self, entry):

        group = {}
        attrs = entry[1]

        group['cn'] = attrs['cn'][0]

        if 'description' in attrs:
            group['description'] = attrs['description'][0]


        return group

    def create_ldap_attrs_from_group(self, user):

        attrs = [
            ('objectclass', [
                'top',
                'groupOfUniqueNames'
            ]),
            ('cn', [group['cn']]),
        ]

        if 'description' in user:
            attrs.append(('description', [group['description']]))

        return attrs

    def get_system_groups(self):

        base_dn = self.config['internaldb.basedn']

        con = self.open_database()

        entries = con.search_s(
            'ou=groups,%s' % base_dn,
            ldap.SCOPE_ONELEVEL,
            '(objectClass=*)',
            None)

        con.unbind_s()

        groups = []
        for entry in entries:
            groups.append(self.create_group_from_ldap_entry(entry))

        return groups

    def get_system_group(self, group_id):

        base_dn = self.config['internaldb.basedn']

        con = self.open_database()

        entries = con.search_s(
            'cn=%s,ou=groups,%s' % (group_id, base_dn),
            ldap.SCOPE_BASE,
            '(objectclass=*)',
            None)

        con.unbind_s()

        entry = entries[0]

        return self.create_group_from_ldap_entry(entry)

    def add_system_group(self, group):

        base_dn = self.config['internaldb.basedn']
        dn = 'cn=%s,ou=groups,%s' % (group['cn'], base_dn)

        attrs = self.create_ldap_attrs_from_group(group)

        con = self.open_database()

        con.add_s(dn, attrs)

        con.unbind_s()

    def remove_system_group(self, group_id):

        base_dn = self.config['internaldb.basedn']

        con = self.open_database()

        con.delete_s(
            'cn=%s,ou=groups,%s' % (group_id, base_dn))

        con.unbind_s()

    def get_system_group_members(self, group_id):

        base_dn = self.config['internaldb.basedn']

        con = self.open_database()

        entries = con.search_s(
            'cn=%s,ou=groups,%s' % (group_id, base_dn),
            ldap.SCOPE_BASE,
            '(objectclass=*)',
            None)

        con.unbind_s()

        entry = entries[0]
        attrs = entry[1]

        if 'uniqueMember' in attrs:
            members = attrs['uniqueMember']
        else:
            members = []

        user_ids = []
        for member_dn in members:
            user_id = ldap.dn.explode_dn(member_dn, notypes=True)[0]
            user_ids.append(user_id)

        return user_ids

    def add_system_group_member(self, group_id, member_id):

        base_dn = self.config['internaldb.basedn']
        group_dn = 'cn=%s,ou=groups,%s' % (group_id, base_dn)
        member_dn = 'uid=%s,ou=people,%s' % (member_id, base_dn)

        modlist = [
            (ldap.MOD_ADD, 'uniqueMember', [member_dn]) 
        ]

        con = self.open_database()

        con.modify_s(group_dn, modlist)

        con.unbind_s()

    def remove_system_group_member(self, group_id, member_id):

        base_dn = self.config['internaldb.basedn']
        group_dn = 'cn=%s,ou=groups,%s' % (group_id, base_dn)
        member_dn = 'uid=%s,ou=people,%s' % (member_id, base_dn)

        modlist = [
            (ldap.MOD_DELETE, 'uniqueMember', [member_dn]) 
        ]

        con = self.open_database()

        con.modify_s(group_dn, modlist)

        con.unbind_s()
