#!/usr/bin/python
# Authors:
#     Endi S. Dewata <edewata@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2015 Red Hat, Inc.
# All rights reserved.
#

import base64
import getopt
import ldap
import os
import sys

import pki.cli
from pki.server import PKISubsystem


class CertCLI(pki.cli.CLI):

    def __init__(self):
        super(CertCLI, self).__init__('cert',
                                      'Certificate management commands')

        self.add_module(CertFindCLI())
        self.add_module(CertShowCLI())

    @staticmethod
    def print_cert(cert, all=False):

        print '  Tag: %s' % cert['tag']
        print '  Nickname: %s' % cert['nickname']
        print '  Version: %s' % cert['version']
        print '  Serial Number: %s' % cert['serial_number']
        print '  Issuer: %s' % cert['issuer']
        print '  Subject: %s' % cert['subject']

        if not all:
            return

        print '  Usage: %s' % cert['usage']
        print '  Token: %s' % cert['token']

        if 'certificate' in cert:
            print '  Certificate: %s' % base64.b64encode(cert['certificate'])

        if 'request' in cert:
            print '  Request: %s' % base64.b64encode(cert['request'])


class CertFindCLI(pki.cli.CLI):

    def __init__(self):
        super(CertFindCLI, self).__init__('find', 'Find certificates')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS]' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, _ = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        certs = subsystem.get_system_certs()

        for cert in certs:
            CertCLI.print_cert(cert)
            print


class CertShowCLI(pki.cli.CLI):

    def __init__(self):
        super(CertShowCLI, self).__init__('show', 'Show certificate')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <certificate tag>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing certificate tag'
            self.print_help()
            sys.exit(1)

        tag = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        cert = subsystem.get_system_cert(tag)

        CertCLI.print_cert(cert, True)
