#!/usr/bin/python
# Authors:
#     Endi S. Dewata <edewata@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2015 Red Hat, Inc.
# All rights reserved.
#

import base64
import getopt
import os
import sys

import pki.cli
from pki.server import PKISubsystem
from pki.server.cli.user import UserCLI


class GroupCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupCLI, self).__init__('group',
                                      'Group management commands')

        self.add_module(GroupFindCLI())
        self.add_module(GroupShowCLI())
        self.add_module(GroupAddCLI())
        self.add_module(GroupRemoveCLI())
        self.add_module(GroupMemberCLI())

    @staticmethod
    def print_group(group, all=False):

        print '  Group ID: %s' % group['cn']

        if 'description' in group:
            print '  Description: %s' % group['description']

        if not all:
            return


class GroupFindCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupFindCLI, self).__init__('find', 'Find groups')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS]' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, _ = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        groups = subsystem.get_system_groups()

        for group in groups:
            GroupCLI.print_group(group)
            print


class GroupShowCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupShowCLI, self).__init__('show', 'Show group')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <group ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing group ID'
            self.print_help()
            sys.exit(1)

        group_id = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        group = subsystem.get_system_group(group_id)

        GroupCLI.print_group(group, True)


class GroupAddCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupAddCLI, self).__init__('add', 'Add group')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <group ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'desc=', 'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing group ID'
            self.print_help()
            sys.exit(1)

        group_id = args[0]
        description = None
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o == '--desc':
                description = a

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        group = {}
        group['cn'] = group_id

        if description:
            group['description'] = description

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.add_system_group(group)


class GroupRemoveCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupRemoveCLI, self).__init__('del', 'Remove group')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <group ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing group ID'
            self.print_help()
            sys.exit(1)

        group_id = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.remove_system_group(group_id)


class GroupMemberCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupMemberCLI, self).__init__('member',
                                      'Group member management commands')

        self.add_module(GroupMemberFindCLI())
        self.add_module(GroupMemberAddCLI())
        self.add_module(GroupMemberRemoveCLI())


class GroupMemberFindCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupMemberFindCLI, self).__init__('find', 'Find group members')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <group ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) < 1:
            print 'ERROR: missing group ID'
            self.print_help()
            sys.exit(1)

        if len(args) > 1:
            print 'ERROR: too many arguments'
            self.print_help()
            sys.exit(1)

        group_id = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        user_ids = subsystem.get_system_group_members(group_id)

        for user_id in user_ids:
            user = subsystem.get_system_user(user_id)
            UserCLI.print_user(user)
            print


class GroupMemberAddCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupMemberAddCLI, self).__init__('add', 'Add group member')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <group ID> <user ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) < 1:
            print 'ERROR: missing group ID'
            self.print_help()
            sys.exit(1)

        if len(args) < 2:
            print 'ERROR: missing user ID'
            self.print_help()
            sys.exit(1)

        if len(args) > 2:
            print 'ERROR: too many arguments'
            self.print_help()
            sys.exit(1)

        group_id = args[0]
        user_id = args[1]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.add_system_group_member(group_id, user_id)


class GroupMemberRemoveCLI(pki.cli.CLI):

    def __init__(self):
        super(GroupMemberRemoveCLI, self).__init__('del', 'Remove group member')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <group ID> <user ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) < 1:
            print 'ERROR: missing group ID'
            self.print_help()
            sys.exit(1)

        if len(args) < 2:
            print 'ERROR: missing user ID'
            self.print_help()
            sys.exit(1)

        if len(args) > 2:
            print 'ERROR: too many arguments'
            self.print_help()
            sys.exit(1)

        group_id = args[0]
        user_id = args[1]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.remove_system_group_member(group_id, user_id)
