#!/usr/bin/python
# Authors:
#     Endi S. Dewata <edewata@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2015 Red Hat, Inc.
# All rights reserved.
#

import base64
import getopt
import os
import sys

import pki.cli
from pki.server import PKISubsystem


class UserCLI(pki.cli.CLI):

    def __init__(self):
        super(UserCLI, self).__init__('user',
                                      'User management commands')

        self.add_module(UserFindCLI())
        self.add_module(UserShowCLI())
        self.add_module(UserAddCLI())
        self.add_module(UserModifyCLI())
        self.add_module(UserDelCLI())

    @staticmethod
    def print_user(user, all=False):

        print '  User ID: %s' % user['uid']
        print '  Common Name: %s' % user['cn']
        print '  Surname: %s' % user['sn']
        print '  Type: %s' % user['type']

        if 'description' in user:
            print '  Description: %s' % user['description']

        if 'mail' in user:
            print '  E-mail: %s' % user['mail']

        if not all:
            return

        if 'certificate' in user:
             print '  Certificate: %s' % base64.b64encode(user['certificate'])

        if 'password' in user:
            print '  Password: %s' % user['password']

        if 'state' in user:
            print '  State: %s' % user['state']


class UserFindCLI(pki.cli.CLI):

    def __init__(self):
        super(UserFindCLI, self).__init__('find', 'Find users')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS]' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '      --desc <description>     Description.'
        print '      --cert <certificate>     Base-64 encoded certificate.'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, _ = getopt.gnu_getopt(argv, 's:v', [
                'desc=', 'cert=',
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        description = None
        certificate = None
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o == '--desc':
                description = a

            elif o == '--cert':
                certificate = base64.b64decode(a)

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        users = subsystem.find_system_users(
            description=description,
            certificate=certificate)

        for user in users:
            UserCLI.print_user(user)
            print


class UserShowCLI(pki.cli.CLI):

    def __init__(self):
        super(UserShowCLI, self).__init__('show', 'Show user')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <user ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing user ID'
            self.print_help()
            sys.exit(1)

        user_id = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        user = subsystem.get_system_user(user_id)

        UserCLI.print_user(user, True)


class UserAddCLI(pki.cli.CLI):

    def __init__(self):
        super(UserAddCLI, self).__init__('add', 'Add user')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <user ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'cn=', 'sn=', 'type=',
                'desc=', 'mail=', 'cert=',
                'password=', 'state=',
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing user ID'
            self.print_help()
            sys.exit(1)

        user_id = args[0]
        cn = user_id
        sn = user_id
        type = 'adminType'
        description = None
        mail = None
        certificate = None
        password = None
        state = None

        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o == '--cn':
                cn = a

            elif o == '--sn':
                sn = a

            elif o == '--type':
                type = a

            elif o == '--desc':
                description = a

            elif o == '--mail':
                mail = a

            elif o == '--cert':
                certificate = a

            elif o == '--password':
                password = a

            elif o == '--state':
                state = a

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        user = {}
        user['uid'] = user_id
        user['cn'] = cn
        user['sn'] = sn
        user['type'] = type

        if description:
            user['description'] = description

        if mail:
            user['mail'] = mail

        if certificate:
            user['certificate'] = base64.b64decode(certificate)

        if password:
            user['password'] = password

        if state:
            user['state'] = state

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.add_system_user(user)


class UserModifyCLI(pki.cli.CLI):

    def __init__(self):
        super(UserModifyCLI, self).__init__('mod', 'Modify user')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <user ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'cn=', 'sn=', 'type=',
                'desc=', 'mail=', 'cert=',
                'password=', 'state=',
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing user ID'
            self.print_help()
            sys.exit(1)

        user_id = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        changes = {}

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o == '--cn':
                changes['cn'] = a

            elif o == '--sn':
                changes['sn'] = a

            elif o == '--type':
                changes['type'] = a

            elif o == '--desc':
                changes['description'] = a

            elif o == '--mail':
                changes['mail'] = a

            elif o == '--cert':
                changes['certificate'] = a

            elif o == '--password':
                changes['password'] = a

            elif o == '--state':
                changes['state'] = a

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.modify_system_user(user_id, changes)


class UserDelCLI(pki.cli.CLI):

    def __init__(self):
        super(UserDelCLI, self).__init__('del', 'Delete user')

    def print_help(self):
        print 'Usage: pki-server %s [OPTIONS] <user ID>' % self.get_full_name()
        print
        print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
        print
        print '  -v, --verbose                Run in verbose mode.'
        print '      --help                   Show help message.'
        print

    def execute(self, argv):

        try:
            opts, args = getopt.gnu_getopt(argv, 's:v', [
                'subsystem=',
                'verbose', 'help'])

        except getopt.GetoptError as e:
            print 'ERROR: ' + str(e)
            self.print_help()
            sys.exit(1)

        if len(args) != 1:
            print 'ERROR: missing user ID'
            self.print_help()
            sys.exit(1)

        user_id = args[0]
        subsystem_dir = '/var/lib/pki-ca'

        for o, a in opts:
            if o in ('-v', '--verbose'):
                self.set_verbose(True)

            elif o == '--help':
                self.print_help()
                sys.exit()

            elif o in ('-s', '--subsystem'):
                subsystem_dir = a

            else:
                print 'ERROR: unknown option ' + o
                self.print_help()
                sys.exit(1)

        subsystem = PKISubsystem(subsystem_dir)

        subsystem.remove_system_user(user_id)
