#!/usr/bin/python
# Authors:
#     Endi S. Dewata <edewata@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2015 Red Hat, Inc.
# All rights reserved.
#

import base64
import getopt
import ldap
import sys

from pki.server import PKISubsystem

def print_help():
    print 'Usage: restore-subsystem-user.py [OPTIONS]'
    print
    print '  -s, --subsystem <directory>  Subsystem directory (default: /var/lib/pki-ca).'
    print
    print '  -v, --verbose                Run in verbose mode.'
    print '      --help                   Show help message.'
    print

try:
    opts, _ = getopt.gnu_getopt(sys.argv, 's:v', [
        'subsystem=',
        'verbose', 'help'])

except getopt.GetoptError as e:
    print 'ERROR: ' + str(e)
    print_help()
    sys.exit(1)

verbose = False
subsystem_dir = '/var/lib/pki-ca'

for o, a in opts:
    if o in ('-v', '--verbose'):
        verbose = True

    elif o == '--help':
        print_help()
        sys.exit()

    elif o in ('-s', '--subsystem'):
        subsystem_dir = a

    else:
        print 'ERROR: unknown option ' + o
        self.print_help()
        sys.exit(1)

print 'Restoring subsystem user'

subsystem = PKISubsystem(subsystem_dir)

type = subsystem.config['cs.type']
hostname = subsystem.config['machineName']
port = subsystem.config['service.securePort']

user_id = '%s-%s-%s' % (type, hostname, port)

cert = subsystem.get_system_cert('subsystem')
version = cert['version']
serial_number = cert['serial_number']
issuer = cert['issuer']
subject = cert['subject']

description = '%s;%s;%s;%s' % (version, serial_number, issuer, subject)
print 'Subsystem certificate: %s' % description

certificate = cert['certificate']
print '-----BEGIN CERTIFICATE-----'
print base64.b64encode(certificate)
print '-----END CERTIFICATE-----'

users = subsystem.find_system_users(certificate=certificate)
members = subsystem.get_system_group_members('Subsystem Group')

for user in users:
    print 'User %s has subsystem certificate' % user['uid']

    if user['uid'] in members:
        print 'User already in Subsystem Group'

        if 'description' in user:
            desc = user['description']
        else:
            desc = None

        if desc == description:
            print 'User has the correct certificate mapping'
            print 'User %s is the subsystem user' % user['uid']

        else:
            changes = {}
            changes['description'] = description
            subsystem.modify_system_user(user_id, changes)
            print 'Certificate mapping fixed'
            print 'Subsystem user %s restored' % user['uid']

        sys.exit()

# If the user with the subsystem certificate doesn't exist,
# or if the user exists but not in the Subsystem Group,
# create a new subsystem user

try:
    user = {}
    user['uid'] = user_id
    user['cn'] = user_id
    user['sn'] = user_id
    user['type'] = 'agentType'
    user['description'] = description
    user['certificate'] = certificate
    user['state'] = '1'

    subsystem.add_system_user(user)
    print 'New subsystem user %s added' % user_id

except ldap.ALREADY_EXISTS:
    print 'Subsystem user %s already exists' % user_id
    user = subsystem.get_system_user(user_id)

    changes = {}
    changes['description'] = description
    changes['certificate'] = certificate
    subsystem.modify_system_user(user_id, changes)
    print 'User certificate updated'

try:
    subsystem.add_system_group_member('Subsystem Group', user_id)
    print 'User added to Subsystem Group'

except ldap.TYPE_OR_VALUE_EXISTS:
    print 'User already in Subsystem Group'

print 'Subsystem user %s restored' % user_id
